==> ec-rider0.8.1.tar.gz: ec-rider0.8.1/ec_front/app/mailers/front_site_mailer.rb <==
# -*- coding: utf-8 -*-
require 'nkf'
require 'cgi'
require 'iconv'
class FrontSiteMailer < ActionMailer::Base

  @@default_charset = 'iso-2022-jp'
  @@encode_subject  = false

  # EC-FRONT側からのメール送信はBCCに送信元を追加する。

  #
  # = ユーザ登録メールの送信 =
  #
  def self.send_signup_mail(user_id, email)
    pc_mobile_div = UserEmail.check_by_emall email
    template = MailTemplate.find_valid_template(:regist_user)
    options = {:user => User.find(user_id.to_i), :email_address => email}
    subject = template.get_mail_subject(pc_mobile_div, options)
    body = template.get_mail_body(pc_mobile_div, options)
    #ユーザ登録メール
    tmail = FrontSiteMailer.deliver_signup(user_id, email)
    #メール配信情報作成
    mail_delivery = MailDelivery.create_front_mail_history(tmail, subject, body, user_id, :regist_user)
    mail_delivery.site_div_key = :jisha
    mail_delivery.save!
  end

  #
  #　= サンクスメールの送信 =
  #
  def self.send_thanks_mail(session_cart_tmp_user, user_logged_in, order_id)
    user = session_cart_tmp_user # || current_user
    email = user.user_email.email
    #非会員ユーザーで会員登録する場合、登録メールも同時送信
    unless user_logged_in
      if user.member_div_key == :membership
        #新規ユーザー登録メール送信
        self.send_signup_mail(user.id.to_s, email)
      end
    end
    pc_mobile_div = UserEmail.check_by_emall email
    order = Order.find(order_id)
    mail_type_div = nil
    order.order_shippings.each do |order_shipping|
      if order_shipping.payment_type_div_key == :bank_transfer
        #銀行振り込み決済が存在する場合は注文受付（銀行振り込み）メール
        mail_type_div = :thanks_bank_transfer
        break
      end
    end
    mail_type_div ||= :thanks
    template = MailTemplate.find_valid_template(mail_type_div)
    options = {:order => order, :user => order.user}
    subject = template.get_mail_subject(pc_mobile_div, options)
    body    = template.get_mail_body(pc_mobile_div, options)

    #THANKSメールの送付
    thanks_tmail = FrontSiteMailer.deliver_thanks(order_id, email)
    #THANKSメール配信情報作成
    thanks_mail_delivery = MailDelivery.create_front_mail_history(thanks_tmail, subject, body, user.id.to_s, :thanks)
    thanks_mail_delivery.site_div_key = :jisha
    thanks_mail_delivery.order_id = order_id
    thanks_mail_delivery.save!
  end

  #
  # = ユーザ退会メールの送信 =
  #
  def self.send_resign_mail(user_id, last_name, first_name, user_name, email)
    pc_mobile_div = UserEmail.check_by_emall email
    template = MailTemplate.find_valid_template(:update_user)
    options = {:last_name => last_name, :first_name => first_name, :user_name => user_name, :email_address => email}
    subject = template.get_mail_subject(pc_mobile_div, options)
    body    = template.get_mail_body(pc_mobile_div, options)
    # 退会完了メールの送信
    tmail = FrontSiteMailer.deliver_resign(last_name, first_name, user_name, email)
    #メール配信情報作成
    mail_delivery = MailDelivery.create_front_mail_history(tmail, subject, body, user_id, :update_user)
    mail_delivery.site_div_key = :jisha
    mail_delivery.save!
  end

  #
  # = キャンセル依頼の通知メール送信 =
  #
  def self.send_notice_cancel_reqest_mail(order, user, email)
    pc_mobile_div = UserEmail.check_by_emall email
    template = MailTemplate.find_valid_template(:cancel)
    options = {:order => order, :user => user}
    subject = template.get_mail_subject(pc_mobile_div, options)
    body    = template.get_mail_body(pc_mobile_div, options)
    #キャンセル依頼の通知メール
    tmail = FrontSiteMailer.deliver_notice_cancel_reqest(order, user, email)
    #メール配信情報作成
    mail_delivery = MailDelivery.create_front_mail_history(tmail, subject, body, user.id, :cancel)
    mail_delivery.site_div_key = :jisha
    mail_delivery.order_id = order.id
    mail_delivery.save!
  end

  #
  # = 問合せ通知メール送信 =
  #
  def self.send_notice_inquiry_mail(inquiry)
    pc_mobile_div = UserEmail.check_by_emall inquiry.email
    template = MailTemplate.find_valid_template(:inquiry)
    options = {:last_name => inquiry.name_last, :first_name => inquiry.name_first, :user_name => inquiry.fullname,
      :inquiry_content => inquiry.inquiry, :email_address => inquiry.email}
    subject = template.get_mail_subject(pc_mobile_div, options)
    body    = template.get_mail_body(pc_mobile_div, options)
    #問合せ通知メール
    tmail = FrontSiteMailer.deliver_notice_inquiry(inquiry)
    #メール配信情報作成
    mail_delivery = MailDelivery.create_front_mail_history(tmail, subject, body, inquiry.user_id, :inquiry)
    mail_delivery.site_div_key = :jisha
    mail_delivery.inquiry_id = inquiry.id
    mail_delivery.save!
  end

  #
  # = 決済取り消しメール送信 =
  #
  def self.send_payment_cancel_request_mail(sps_parameter_history, sps_result_history, user = User.new() ,error_reason = "")
    # options[;user]は注文者を設定
    options = {
      :user => user,
      :payment_error_reason => error_reason,
      :payment_error_datetime => Date.today,
      :sps_parameter_history => sps_parameter_history,
      :sps_result_history => sps_result_history
    }
    template = MailTemplate.find_valid_template(:cancel_payment)
    # TODO 管理者宛てのためpc,日本語のみ
    subject = template.get_mail_subject(:pc, options)
    body = template.get_mail_body(:pc, options)
    to_list = ShopSetting[:payment_cancel_mail].split(/\s*,\s*/)
    tmail = FrontSiteMailer.deliver_payment_cancel_request(to_list, subject, body)

    #メール配信履歴情報作成
    mail_deliverie = MailDelivery.create_front_mail_history(tmail, subject, body, nil, :cancel_payment)
    mail_deliverie.site_div_key = :jisha
    mail_deliverie.save!
  end

  #
  # = ユーザ登録メール =
  #
  def signup(user_id, email, sent_at = Time.now)
    pc_mobile_div = UserEmail.check_by_emall email
    template = MailTemplate.find_valid_template(:regist_user)
    options = {:user => User.find(user_id.to_i), :email_address => email}

    @charset = locale_to_charset(template.locale)
    @subject    = template.get_mail_subject(pc_mobile_div, options)

    @body       = {:body => template.get_mail_body(pc_mobile_div, options)}
    @recipients = email
    @from       = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @bcc        = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @sent_on    = sent_at
    @headers    = {}
  end

  #
  # = ユーザ退会通知メール =
  #
  def resign(last_name, first_name, user_name, email, sent_at = Time.now)
    pc_mobile_div = UserEmail.check_by_emall email
    template = MailTemplate.find_valid_template(:update_user)
    options = {:last_name => last_name, :first_name => first_name, :user_name => user_name, :email_address => email}

    @charset = locale_to_charset(template.locale)
    @subject    = template.get_mail_subject(pc_mobile_div, options)
    @body       = {:body => template.get_mail_body(pc_mobile_div, options)}
    @recipients = email
    @from       = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @bcc        = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @sent_on    = sent_at
    @headers    = {}
  end

  #
  # = サンクスメール =
  #
  def thanks(order_id, email, sent_at = Time.now)
    pc_mobile_div = UserEmail.check_by_emall email
    order = Order.find(order_id)
    mail_type_div = nil
    order.order_shippings.each do |order_shipping|
      if order_shipping.payment_type_div_key == :bank_transfer
        #銀行振り込み決済が存在する場合は注文受付（銀行振り込み）メール
        mail_type_div = :thanks_bank_transfer
        break
      end
    end
    mail_type_div ||= :thanks
    template = MailTemplate.find_valid_template(mail_type_div)
    options = {:order => order, :user => order.user}

    @charset = locale_to_charset(template.locale)
    @subject    = template.get_mail_subject(pc_mobile_div, options)
    @body       = {:body => template.get_mail_body(pc_mobile_div, options)}
    @recipients = email
    @from       = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @bcc        = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @sent_on    = sent_at
    @headers    = {}
  end

  #
  # = キャンセル依頼の通知メール =
  #
  def notice_cancel_reqest(order, user, email, sent_at = Time.now)
    pc_mobile_div = UserEmail.check_by_emall email
    template = MailTemplate.find_valid_template(:cancel)

    options = {:order => order, :user => user}

    @charset = locale_to_charset(template.locale)
    @subject    = template.get_mail_subject(pc_mobile_div, options)
    @body       = {:body => template.get_mail_body(pc_mobile_div, options)}
    @recipients = email
    @from       = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @bcc        = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @sent_on    = sent_at
    @headers    = {}
  end

  #
  # = 問合せ通知メール =
  #
  def notice_inquiry(inquiry, sent_at = Time.now)
    pc_mobile_div = UserEmail.check_by_emall inquiry.email
    template = MailTemplate.find_valid_template(:inquiry)
    options = {:last_name => inquiry.name_last, :first_name => inquiry.name_first, :user_name => inquiry.fullname,
      :inquiry_content => inquiry.inquiry, :email_address => inquiry.email}

    # 商品お問合せの場合はカテゴリの担当者をCCに入れる
    bcc_list = [EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]]

    if (inquiry.inquiry_div_key == :product && !inquiry.inquiry_products.empty?)
      inquiry_products = inquiry.inquiry_products
      product = Product.find(inquiry_products[0].product_id)
      bcc_list.concat(ProductCategory.get_staff_emails_for_cc(product))
    end

    @charset = locale_to_charset(template.locale)
    @subject    = template.get_mail_subject(pc_mobile_div, options)
    @body       = {:body => template.get_mail_body(pc_mobile_div, options)}
    @recipients = inquiry.email
    @from       = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @bcc        = bcc_list
    @sent_on    = sent_at
    @headers    = {}
  end

  #
  # = キャンセル依頼の通知メール =
  #
  def payment_cancel_request(to_list, subject, body)
    @charset = locale_to_charset(:ja)
    @subject    = subject
    @body       = {:body => body}
    @recipients = to_list
    @from       = ShopSetting[:shop_mail]
    @bcc        = ShopSetting[:shop_mail]
    @sent_on    = Time.now
    @headers    = {}
  end

  private
  #
  # = BASE64エンコード変換 =
  #  対応キャラクタセットはiso-2022-jp, iso-8859-1, GB2312, Big5。
  #  エンコード失敗時に例外Iconv::Failureが発生します。
  #
  def base64(text, charset="iso-2022-jp", convert=true)
    charset = charset.downcase unless charset.nil?
    if convert
      if charset == 'iso-2022-jp' || charset == 'iso-8859-1' || charset == 'gb2312' || charset == 'big5'
        text = Iconv.conv(charset, 'utf-8', text)
      end
    end
    text = [text].pack('m').delete("\r\n")
    "=?#{charset}?B?#{text}?="
  end

  #
  # = ロケール文字列→キャラクタセット文字列変換 =
  #
  def locale_to_charset(locale)
    case locale
    when 'en'
      return 'iso-8859-1'
    when 'zh_CN'
      return "gb2312"
    when 'zh_TW'
      return "big5"
    else
      return "iso-2022-jp"
    end
  end

  #
  # = 本文の文字コード変換 =
  # @localeの内容に応じて自動的に変換文字コードを判別します。
  # @localeがnilの場合はiso-2022-jpで送信します。（デフォルト動作）
  #
  #def create! (*)
  #  super
  #  @mail.subject = base64(@subject, charset)
  #  @mail.body = NKF::nkf('-j', @mail.body) or Iconv.conv(charset, 'UTF-8', part.body)
  #  return @mail
  #end
  def create! (*)
    super #@mailの生成
    part = @mail.parts.empty? ? @mail : @mail.parts.first
    retry_charset = 'utf-8' #エンコードに失敗した場合はUTF-8(無変換)で送信する
    begin #Subjectのエンコード
      @mail.subject = base64(@subject, @charset)
    rescue Iconv::Failure
      @charset = retry_charset
      retry
    end
    is_retry = false #本文エンコード失敗時にtrue
    begin #本文のエンコード
      if part.content_type == "text/plain"
        if @charset == 'utf-8'
          @mail.subject = base64(@subject, @charset, false) if is_retry
          part.charset = charset
        elsif @charset == 'iso-8859-1'
          part.charset = charset
          part.body = Iconv.conv(charset, 'utf-8', part.body)
          part.content_transfer_encoding = '7bit'
        elsif @charset == 'gb2312' || @charset == 'big5'
          part.charset = charset
          part.body = Iconv.conv(charset, 'utf-8', part.body)
        else
          #default
          part.charset = 'iso-2022-jp'
          part.body = Iconv.conv(charset, 'utf-8', part.body)
          part.content_transfer_encoding = '7bit'  # 申請の「請」などが無くなってしまう。
        end
      end
    rescue Iconv::Failure
      @charset = retry_charset
      is_retry = true
      retry
    end
    return @mail   # メソッドチェインを期待した変更があったら怖いので
  end
end

==> ec-rider0.8.2.tar.gz: ec-rider0.8.2/ec_front/app/mailers/front_site_mailer.rb <==
# -*- coding: utf-8 -*-
require 'nkf'
require 'cgi'
require 'iconv'
class FrontSiteMailer < ActionMailer::Base
  unloadable
  include EcRider::PluginModule::Extendable

  @@default_charset = 'iso-2022-jp'
  @@encode_subject  = false

  # EC-FRONT側からのメール送信はBCCに送信元を追加する。

  #
  # = ユーザ登録メールの送信 =
  #
  def self.send_signup_mail(user_id, email)
    pc_mobile_div = UserEmail.check_by_emall email
    template = MailTemplate.find_valid_template(:regist_user)
    options = {:user => User.find(user_id.to_i), :email_address => email}
    subject = template.get_mail_subject(pc_mobile_div, options)
    body = template.get_mail_body(pc_mobile_div, options)
    #ユーザ登録メール
    tmail = FrontSiteMailer.deliver_signup(user_id, email)
    #メール配信情報作成
    mail_delivery = MailDelivery.create_front_mail_history(tmail, subject, body, user_id, :regist_user)
    mail_delivery.site_div_key = :jisha
    mail_delivery.save!
  end

  #
  # = ユーザ登録エラーメールの送信 =
  #
  def self.send_signup_error_mail(user_id, email)
    pc_mobile_div = UserEmail.check_by_emall email
    template = MailTemplate.find_valid_template(:regist_user_error)
    options = {:user => User.find(user_id.to_i), :email_address => email}
    subject = template.get_mail_subject(pc_mobile_div, options)
    body = template.get_mail_body(pc_mobile_div, options)
    #ユーザ登録エラーメール
    tmail = FrontSiteMailer.deliver_signup_error(user_id, email)
    #メール配信情報作成
    mail_delivery = MailDelivery.create_front_mail_history(tmail, subject, body, user_id, :regist_user_error)
    mail_delivery.site_div_key = :jisha
    mail_delivery.save!
  end

  #
  #　= 注文完了後メールの一括送信メソッド =
  # 注文完了後に以下のメールを送信します。
  # 注文ありがとうございましたメール
  # (新規会員登録する場合)ユーザ登録メール
  # (新規会員登録に失敗した場合)ユーザ登録エラーメール
  def self.send_thanks_mail(session_cart_tmp_user, user_logged_in, order_id)
    user = session_cart_tmp_user # || current_user
    order = Order.find(order_id)
    email = order.site_type_div_key == :pc ? user.user_email.email : user.user_email_mobile.email
    #非会員ユーザーで会員登録する場合、登録メールも同時送信
    unless user_logged_in
      if user.member_div_key == :membership
        #新規ユーザー登録メール送信
        self.send_signup_mail(user.id.to_s, email)
      elsif (user.member_div_key == :no_membership && !user.regist_error.nil?)
        # 会員登録失敗した会員には
        # 会員登録ができなかった旨のメールを送信
        self.send_signup_error_mail(user.id.to_s, email)
      end
    end
    pc_mobile_div = UserEmail.check_by_emall email
    mail_type_div = nil
    order.order_shippings.each do |order_shipping|
      if order_shipping.payment_type_div_key == :bank_transfer
        #銀行振り込み決済が存在する場合は注文受付（銀行振り込み）メール
        mail_type_div = :thanks_bank_transfer
        break
      end
    end
    mail_type_div ||= :thanks
    template = MailTemplate.find_valid_template(mail_type_div)
    options = {:order => order, :user => order.user}
    subject = template.get_mail_subject(pc_mobile_div, options)
    body    = template.get_mail_body(pc_mobile_div, options)

    #THANKSメールの送付
    thanks_tmail = FrontSiteMailer.deliver_thanks(order_id, email)
    #THANKSメール配信情報作成
    thanks_mail_delivery = MailDelivery.create_front_mail_history(thanks_tmail, subject, body, user.id.to_s, :thanks)
    thanks_mail_delivery.site_div_key = :jisha
    thanks_mail_delivery.order_id = order_id
    thanks_mail_delivery.save!
  end

  #
  # = ユーザ退会メールの送信 =
  #
  def self.send_resign_mail(user_id, last_name, first_name, user_name, email)
    pc_mobile_div = UserEmail.check_by_emall email
    template = MailTemplate.find_valid_template(:update_user)
    options = {:last_name => last_name, :first_name => first_name, :user_name => user_name, :email_address => email}
    subject = template.get_mail_subject(pc_mobile_div, options)
    body    = template.get_mail_body(pc_mobile_div, options)
    # 退会完了メールの送信
    tmail = FrontSiteMailer.deliver_resign(last_name, first_name, user_name, email)
    #メール配信情報作成
    mail_delivery = MailDelivery.create_front_mail_history(tmail, subject, body, user_id, :update_user)
    mail_delivery.site_div_key = :jisha
    mail_delivery.save!
  end

  #
  # = キャンセル依頼の通知メール送信 =
  #
  def self.send_notice_cancel_reqest_mail(order, user, email)
    pc_mobile_div = UserEmail.check_by_emall email
    template = MailTemplate.find_valid_template(:cancel)
    options = {:order => order, :user => user}
    subject = template.get_mail_subject(pc_mobile_div, options)
    body    = template.get_mail_body(pc_mobile_div, options)
    #キャンセル依頼の通知メール
    tmail = FrontSiteMailer.deliver_notice_cancel_reqest(order, user, email)
    #メール配信情報作成
    mail_delivery = MailDelivery.create_front_mail_history(tmail, subject, body, user.id, :cancel)
    mail_delivery.site_div_key = :jisha
    mail_delivery.order_id = order.id
    mail_delivery.save!
  end

  #
  # = 問合せ通知メール送信 =
  #
  def self.send_notice_inquiry_mail(inquiry)
    pc_mobile_div = UserEmail.check_by_emall inquiry.email
    template = MailTemplate.find_valid_template(:inquiry)
    options = {:last_name => inquiry.name_last, :first_name => inquiry.name_first, :user_name => inquiry.fullname,
      :inquiry_content => inquiry.inquiry, :email_address => inquiry.email}
    subject = template.get_mail_subject(pc_mobile_div, options)
    body    = template.get_mail_body(pc_mobile_div, options)
    #問合せ通知メール
    tmail = FrontSiteMailer.deliver_notice_inquiry(inquiry)
    #メール配信情報作成
    mail_delivery = MailDelivery.create_front_mail_history(tmail, subject, body, inquiry.user_id, :inquiry)
    mail_delivery.site_div_key = :jisha
    mail_delivery.inquiry_id = inquiry.id
    mail_delivery.save!
  end

  #
  # = ユーザ登録メール =
  #
  def signup(user_id, email, sent_at = Time.now)
    pc_mobile_div = UserEmail.check_by_emall email
    template = MailTemplate.find_valid_template(:regist_user)
    options = {:user => User.find(user_id.to_i), :email_address => email}

    @charset = locale_to_charset(template.locale)
    @subject    = template.get_mail_subject(pc_mobile_div, options)

    @body       = {:body => template.get_mail_body(pc_mobile_div, options)}
    @recipients = email
    @from       = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @bcc        = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @sent_on    = sent_at
    @headers    = {}
  end

  #
  # = ユーザ登録エラーメール =
  #
  def signup_error(user_id, email, sent_at = Time.now)
    pc_mobile_div = UserEmail.check_by_emall email
    template = MailTemplate.find_valid_template(:regist_user_error)
    options = {:user => User.find(user_id.to_i), :email_address => email}

    @charset = locale_to_charset(template.locale)
    @subject    = template.get_mail_subject(pc_mobile_div, options)

    @body       = {:body => template.get_mail_body(pc_mobile_div, options)}
    @recipients = email
    @from       = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @bcc        = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @sent_on    = sent_at
    @headers    = {}
  end

  #
  # = ユーザ退会通知メール =
  #
  def resign(last_name, first_name, user_name, email, sent_at = Time.now)
    pc_mobile_div = UserEmail.check_by_emall email
    template = MailTemplate.find_valid_template(:update_user)
    options = {:last_name => last_name, :first_name => first_name, :user_name => user_name, :email_address => email}

    @charset = locale_to_charset(template.locale)
    @subject    = template.get_mail_subject(pc_mobile_div, options)
    @body       = {:body => template.get_mail_body(pc_mobile_div, options)}
    @recipients = email
    @from       = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @bcc        = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @sent_on    = sent_at
    @headers    = {}
  end

  #
  # = サンクスメール =
  #
  def thanks(order_id, email, sent_at = Time.now)
    pc_mobile_div = UserEmail.check_by_emall email
    order = Order.find(order_id)
    mail_type_div = nil
    order.order_shippings.each do |order_shipping|
      if order_shipping.payment_type_div_key == :bank_transfer
        #銀行振り込み決済が存在する場合は注文受付（銀行振り込み）メール
        mail_type_div = :thanks_bank_transfer
        break
      end
    end
    mail_type_div ||= :thanks
    template = MailTemplate.find_valid_template(mail_type_div)
    options = {:order => order, :user => order.user}

    @charset = locale_to_charset(template.locale)
    @subject    = template.get_mail_subject(pc_mobile_div, options)
    @body       = {:body => template.get_mail_body(pc_mobile_div, options)}
    @recipients = email
    @from       = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @bcc        = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @sent_on    = sent_at
    @headers    = {}
  end

  #
  # = キャンセル依頼の通知メール =
  #
  def notice_cancel_reqest(order, user, email, sent_at = Time.now)
    pc_mobile_div = UserEmail.check_by_emall email
    template = MailTemplate.find_valid_template(:cancel)

    options = {:order => order, :user => user}

    @charset = locale_to_charset(template.locale)
    @subject    = template.get_mail_subject(pc_mobile_div, options)
    @body       = {:body => template.get_mail_body(pc_mobile_div, options)}
    @recipients = email
    @from       = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @bcc        = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @sent_on    = sent_at
    @headers    = {}
  end

  #
  # = 問合せ通知メール =
  #
  def notice_inquiry(inquiry, sent_at = Time.now)
    pc_mobile_div = UserEmail.check_by_emall inquiry.email
    template = MailTemplate.find_valid_template(:inquiry)
    options = {:last_name => inquiry.name_last, :first_name => inquiry.name_first, :user_name => inquiry.fullname,
      :inquiry_content => inquiry.inquiry, :email_address => inquiry.email}

    # 商品お問合せの場合はカテゴリの担当者をCCに入れる
    bcc_list = [EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]]

    if (inquiry.inquiry_div_key == :product && !inquiry.inquiry_products.empty?)
      inquiry_products = inquiry.inquiry_products
      product = Product.find(inquiry_products[0].product_id)
      bcc_list.concat(ProductCategory.get_staff_emails_for_cc(product))
    end

    @charset = locale_to_charset(template.locale)
    @subject    = template.get_mail_subject(pc_mobile_div, options)
    @body       = {:body => template.get_mail_body(pc_mobile_div, options)}
    @recipients = inquiry.email
    @from       = EcRider.config.ec_test ? ShopSetting[:shop_mail_test] : ShopSetting[:shop_mail]
    @bcc        = bcc_list
    @sent_on    = sent_at
    @headers    = {}
  end

  # TODO
  # ec_frontからOrderShipping#send_receipt_payment_mail（入金確認メール送信）を呼ぶ必要があったため、AdminMailerのメソッドを追加。
  # 今後もこのようなことが発生しそうなので、AdminMailerとFrontSiteMailerを統合したほうがよいです。
  #
  # = フリーメール送信 =
  #
  def self.send_free_mail(mail_type_div, from, to, subject, body, locale, user=nil, order=nil, inquiry_id=nil)
    #フリーメール
    email = FrontSiteMailer.deliver_free_mail(from, to, subject, body, locale)
    mail_address = UserEmail.find_by_email(to)
    user ||= mail_address.user unless mail_address.blank?
    #メール配信情報作成
    mail_delivery = MailDelivery.create_front_mail_history(email, subject, body, (user.nil? ? nil : user.id ), mail_type_div)
    mail_delivery.site_div_key = :jisha
    mail_delivery.order_id = order.id if order
    mail_delivery.inquiry_id = inquiry_id if inquiry_id
    mail_delivery.save!
  end
  
  #
  # = フリーメール =
  #
  def free_mail(from, recipients, subject, body, locale, sent_at = Time.now)
    @charset = locale_to_charset(locale)
    @subject    = subject
    @body       = {:body => body}
    @recipients = recipients
    @from       = from
    @bcc        = from
    @sent_on    = sent_at
    @headers    = {}
  end

  private
  #
  # = BASE64エンコード変換 =
  #  対応キャラクタセットはiso-2022-jp, iso-8859-1, GB2312, Big5。
  #  エンコード失敗時に例外Iconv::Failureが発生します。
  #
  def base64(text, charset="iso-2022-jp", convert=true)
    charset = charset.downcase unless charset.nil?
    if convert
      if charset == 'iso-2022-jp' || charset == 'iso-8859-1' || charset == 'gb2312' || charset == 'big5'
        text = Iconv.conv(charset, 'utf-8', text)
      end
    end
    text = [text].pack('m').delete("\r\n")
    "=?#{charset}?B?#{text}?="
  end

  #
  # = ロケール文字列→キャラクタセット文字列変換 =
  #
  def locale_to_charset(locale)
    case locale
    when 'en'
      return 'iso-8859-1'
    when 'zh_CN'
      return "gb2312"
    when 'zh_TW'
      return "big5"
    else
      return "iso-2022-jp"
    end
  end

  #
  # = 本文の文字コード変換 =
  # @localeの内容に応じて自動的に変換文字コードを判別します。
  # @localeがnilの場合はiso-2022-jpで送信します。（デフォルト動作）
  #
  #def create! (*)
  #  super
  #  @mail.subject = base64(@subject, charset)
  #  @mail.body = NKF::nkf('-j', @mail.body) or Iconv.conv(charset, 'UTF-8', part.body)
  #  return @mail
  #end
  def create! (*)
    super #@mailの生成
    part = @mail.parts.empty? ? @mail : @mail.parts.first
    retry_charset = 'utf-8' #エンコードに失敗した場合はUTF-8(無変換)で送信する
    begin #Subjectのエンコード
      @mail.subject = base64(@subject, @charset)
    rescue Iconv::Failure
      @charset = retry_charset
      retry
    end
    is_retry = false #本文エンコード失敗時にtrue
    begin #本文のエンコード
      if part.content_type == "text/plain"
        if @charset == 'utf-8'
          @mail.subject = base64(@subject, @charset, false) if is_retry
          part.charset = charset
        elsif @charset == 'iso-8859-1'
          part.charset = charset
          part.body = Iconv.conv(charset, 'utf-8', part.body)
          part.content_transfer_encoding = '7bit'
        elsif @charset == 'gb2312' || @charset == 'big5'
          part.charset = charset
          part.body = Iconv.conv(charset, 'utf-8', part.body)
        else
          #default
          part.charset = 'iso-2022-jp'
          part.body = Iconv.conv(charset, 'utf-8', part.body)
          part.content_transfer_encoding = '7bit'  # 申請の「請」などが無くなってしまう。
        end
      end
    rescue Iconv::Failure
      @charset = retry_charset
      is_retry = true
      retry
    end
    return @mail   # メソッドチェインを期待した変更があったら怖いので
  end
end
